// 
// File:          gov_cca_AbstractFramework.hxx
// Symbol:        gov.cca.AbstractFramework-v0.8.5
// Symbol Type:   interface
// Babel Version: 1.4.0
// Description:   Client-side glue code for gov.cca.AbstractFramework
// 
// WARNING: Automatically generated; changes will be lost
// 
// 

#ifndef included_gov_cca_AbstractFramework_hxx
#define included_gov_cca_AbstractFramework_hxx

#ifndef included_sidl_cxx_hxx
#include "sidl_cxx.hxx"
#endif
// declare class before main #includes
// (this alleviates circular #include guard problems)[BUG#393]
namespace gov { 
  namespace cca { 

    class AbstractFramework;
  } // end namespace cca
} // end namespace gov

// Some compilers need to define array template before the specializations
namespace sidl {
  template<>
  class array< ::gov::cca::AbstractFramework >;
}
// 
// Forward declarations for method dependencies.
// 
namespace gov { 
  namespace cca { 

    class AbstractFramework;
  } // end namespace cca
} // end namespace gov

namespace gov { 
  namespace cca { 

    class CCAException;
  } // end namespace cca
} // end namespace gov

namespace gov { 
  namespace cca { 

    class Services;
  } // end namespace cca
} // end namespace gov

namespace gov { 
  namespace cca { 

    class TypeMap;
  } // end namespace cca
} // end namespace gov

namespace sidl { 

  class RuntimeException;
} // end namespace sidl

#ifndef included_sidl_cxx_hxx
#include "sidl_cxx.hxx"
#endif
#ifndef included_gov_cca_AbstractFramework_IOR_h
#include "gov_cca_AbstractFramework_IOR.h"
#endif
#ifndef included_sidl_BaseInterface_hxx
#include "sidl_BaseInterface.hxx"
#endif
namespace sidl {
  namespace rmi {
    class Call;
    class Return;
    class Ticket;
  }
  namespace rmi {
    class InstanceHandle;
  }
}
namespace gov { 
  namespace cca { 

    /**
     * Symbol "gov.cca.AbstractFramework" (version 0.8.5)
     * 
     *  
     * This is an interface presented by a CCA-compliant framework to access its 
     * application framing capabilities. Most of the manipulation of the 
     * underlying framework is expected to be be done with the 
     * gov.cca.BuilderService Port. This class exists as a sort of bootstrap 
     * to get a Services object necessary to retrieve Port's, including 
     * BuilderService, from the underlying framework. How the interface and 
     * the underlying framework is created is entirely unspecified and is up 
     * to the devices of the programmer and the framework provider.
     * 
     * <p>Example</p>
     * <p>
     * Here it is assumed that an instance of AbstractFramework
     * is created in the main() from some hypothetical implementation.
     * The idea is to allow a complete swap of framework choice by 
     * changing out the specified implementation class of a framework.
     * </p>
     * 
     * <code><pre>
     * // java
     * main() {
     * cca.reference.Framework fwkimpl = new cca.reference.Framework();
     * // change fwkimpl above to use different cca implementations when
     * // AbstractFramework becomes part of the standard.
     * gov.cca.AbstractFramework fwk = (gov.cca.AbstractFramework)fwkimpl;
     * gov.cca.Services svc = 
     * fwk.getServices("instance0","AppDriver",null);
     * // From here on, access all services, components, etc
     * // through svc.
     * ...
     * // when done
     * fwk.releaseServices(svc);
     * fwk.shutdownFramework();
     * }
     * 
     * // c++
     * int functionName() {
     * ::gov::sandia::ccafe::Framework fwkimpl;
     * ::gov::cca::AbstractFrameworkPtr fwk;
     * 
     * fwk = fwkimpl.getStandardFramework();
     * ::gov::cca::Services_Interface  svc = 0;
     * svc = fwk->getServices("instance0","AppDriver",0);
     * // From here on, access all services, components, etc
     * // through svc.
     * ...
     * // when done
     * fwk->releaseServices(svc);
     * svc = 0;
     * fwk->shutdownFramework();
     * 
     * // at scope exit, all memory is automatically cleaned up.
     * }
     * </pre></code>
     */
    class AbstractFramework: public virtual ::sidl::BaseInterface {

      //////////////////////////////////////////////////
      // 
      // Special methods for throwing exceptions
      // 

    private:
      static 
      void
      throwException1(
        const char* methodName,
        struct sidl_BaseInterface__object *_exception
      )
        // throws:
      ;
      static 
      void
      throwException0(
        const char* methodName,
        struct sidl_BaseInterface__object *_exception
      )
        // throws:
        //    ::gov::cca::CCAException
        //    ::sidl::RuntimeException
      ;

      //////////////////////////////////////////////////
      // 
      // User Defined Methods
      // 

    public:

      /**
       *  
       * Create an empty TypeMap. Presumably this would be used in 
       * an ensuing call to <code>getServices()</code>. The "normal" method of
       * creating typemaps is found in the <code>Services</code> interface. It
       * is duplicated here to break the "chicken and egg" problem.
       */
      ::gov::cca::TypeMap
      createTypeMap() // throws:
      //    ::gov::cca::CCAException
      //    ::sidl::RuntimeException
      ;


      /**
       *  
       * Retrieve a Services handle to the underlying framework. 
       * This interface effectively causes the calling program to 
       * appear as the image of a component inside the framework.
       * This method may be called any number of times
       * with different arguments, creating a new component image 
       * each time. 
       * The only proper method to destroy a Services obtained 
       * from this interface is to pass it to releaseServices.
       * 
       * @param selfInstanceName the Component instance name,
       * as it will appear in the framework.
       * 
       * @param selfClassName the Component type of the 
       * calling program, as it will appear in the framework. 
       * 
       * @param selfProperties (which can be null) the properties 
       * of the component image to appear. 
       * 
       * @throws CCAException in the event that selfInstanceName 
       * is already in use by another component.
       * 
       * @return  A Services object that pertains to the
       * image of the this component. This is identical
       * to the object passed into Component.setServices() 
       * when a component is created.
       */
      ::gov::cca::Services
      getServices (
        /* in */const ::std::string& selfInstanceName,
        /* in */const ::std::string& selfClassName,
        /* in */const ::gov::cca::TypeMap& selfProperties
      )
      // throws:
      //    ::gov::cca::CCAException
      //    ::sidl::RuntimeException
      ;



      /**
       *  
       * Inform framework that the <code>Services</code> handle is no longer needed by the 
       * caller and that the reference to its component image is to be
       * deleted from the context of the underlying framework. This invalidates
       * any <code>ComponentID</code>'s or <code>ConnectionID</code>'s associated 
       * with the given <code>Services</code>' component image. 
       * 
       * @param services The result of getServices earlier obtained.
       * 
       * @throws CCAException if the <code>Services</code>
       * handle has already been released or is otherwise rendered invalid 
       * or was not obtained from <code>getServices()</code>.
       */
      void
      releaseServices (
        /* in */const ::gov::cca::Services& services
      )
      // throws:
      //    ::gov::cca::CCAException
      //    ::sidl::RuntimeException
      ;



      /**
       *  
       * Tell the framework it is no longer needed and to clean up after itself. 
       * @throws CCAException if the framework has already been shutdown.
       */
      void
      shutdownFramework() // throws:
      //    ::gov::cca::CCAException
      //    ::sidl::RuntimeException
      ;


      /**
       *  
       * Creates a new framework instance based on the same underlying 
       * framework implementation. This does not copy the existing 
       * framework, nor are any of the user-instantiated components in
       * the original framework available in the newly created 
       * <code>AbstractFramework</code>. 
       * 
       * @throws CCAException when one of the following conditions occur:
       * 
       * (1)the AbstractFramework previously had shutdownFramework() called on it, or 
       * (2)the underlying framework implementation does not permit creation 
       * of another instance.	 
       */
      ::gov::cca::AbstractFramework
      createEmptyFramework() // throws:
      //    ::gov::cca::CCAException
      //    ::sidl::RuntimeException
      ;


      //////////////////////////////////////////////////
      // 
      // End User Defined Methods
      // (everything else in this file is specific to
      //  Babel's C++ bindings)
      // 

    public:
      typedef struct gov_cca_AbstractFramework__object ior_t;
      typedef struct gov_cca_AbstractFramework__external ext_t;
      typedef struct gov_cca_AbstractFramework__sepv sepv_t;

      // default constructor
      AbstractFramework()  : gov_cca_AbstractFramework_IORCache((ior_t*) NULL){ 
        }

#ifdef WITH_RMI

      // RMI connect
      static inline ::gov::cca::AbstractFramework _connect( /*in*/ const 
        std::string& url ) { 
        return _connect(url, true);
      }

      // RMI connect 2
      static ::gov::cca::AbstractFramework _connect( /*in*/ const std::string& 
        url, /*in*/ const bool ar  );


#endif /*WITH_RMI*/

      // default destructor
      virtual ~AbstractFramework () { }

      // copy constructor
      AbstractFramework ( const AbstractFramework& original );

      // assignment operator
      AbstractFramework& operator= ( const AbstractFramework& rhs );

      // conversion from ior to C++ class
      AbstractFramework ( AbstractFramework::ior_t* ior );

      // Alternate constructor: does not call addRef()
      // (sets d_weak_reference=isWeak)
      // For internal use by Impls (fixes bug#275)
      AbstractFramework ( AbstractFramework::ior_t* ior, bool isWeak );

      inline ior_t* _get_ior() const throw() {
        if(!gov_cca_AbstractFramework_IORCache) { 
          gov_cca_AbstractFramework_IORCache = 
            ::gov::cca::AbstractFramework::_cast((void*)d_self);
          if (gov_cca_AbstractFramework_IORCache) {
            struct sidl_BaseInterface__object *throwaway_exception;
            (gov_cca_AbstractFramework_IORCache->d_epv->f_deleteRef)(
              gov_cca_AbstractFramework_IORCache->d_object, 
              &throwaway_exception);  
          }  
        }
        return gov_cca_AbstractFramework_IORCache;
      }

      inline void _set_ior( ior_t* ptr ) throw () { 
        d_self = reinterpret_cast< void*>(ptr);
        gov_cca_AbstractFramework_IORCache = (ior_t*) ptr;

      }

      virtual int _set_ior_typesafe( struct sidl_BaseInterface__object *obj,
                                     const ::std::type_info &argtype );

      bool _is_nil() const throw () { return (d_self==0); }

      bool _not_nil() const throw () { return (d_self!=0); }

      bool operator !() const throw () { return (d_self==0); }

      static inline const char * type_name() throw () { return 
        "gov.cca.AbstractFramework";}

      static struct gov_cca_AbstractFramework__object* _cast(const void* src);

      // execute member function by name
      void _exec(const std::string& methodName,
                 ::sidl::rmi::Call& inArgs,
                 ::sidl::rmi::Return& outArgs);

      /**
       * Get the URL of the Implementation of this object (for RMI)
       */
      ::std::string
      _getURL() // throws:
      //    ::sidl::RuntimeException
      ;


      /**
       * Method to enable/disable method hooks invocation.
       */
      void
      _set_hooks (
        /* in */bool enable
      )
      // throws:
      //    ::sidl::RuntimeException
      ;


      /**
       * Method to enable/disable interface contract enforcement.
       */
      void
      _set_contracts (
        /* in */bool enable,
        /* in */const ::std::string& enfFilename,
        /* in */bool resetCounters
      )
      // throws:
      //    ::sidl::RuntimeException
      ;


      /**
       * Method to dump contract enforcement statistics.
       */
      void
      _dump_stats (
        /* in */const ::std::string& filename,
        /* in */const ::std::string& prefix
      )
      // throws:
      //    ::sidl::RuntimeException
      ;

      // return true iff object is remote
      bool _isRemote() const { 
        ior_t* self = const_cast<ior_t*>(_get_ior() );
        struct sidl_BaseInterface__object *throwaway_exception;
        return (*self->d_epv->f__isRemote)(self, &throwaway_exception) == TRUE;
      }

      // return true iff object is local
      bool _isLocal() const {
        return !_isRemote();
      }

    protected:
      // Pointer to external (DLL loadable) symbols (shared among instances)
      static const ext_t * s_ext;

    public:
      static const ext_t * _get_ext() throw ( ::sidl::NullIORException );


      //////////////////////////////////////////////////
      // 
      // Locally Cached IOR pointer
      // 

    protected:
      mutable ior_t* gov_cca_AbstractFramework_IORCache;
    }; // end class AbstractFramework
  } // end namespace cca
} // end namespace gov

extern "C" {


#pragma weak gov_cca_AbstractFramework__connectI

  /**
   * RMI connector function for the class. (no addref)
   */
  struct gov_cca_AbstractFramework__object*
  gov_cca_AbstractFramework__connectI(const char * url, sidl_bool ar, struct 
    sidl_BaseInterface__object **_ex);


} // end extern "C"
namespace sidl {
  // traits specialization
  template<>
  struct array_traits< ::gov::cca::AbstractFramework > {
    typedef array< ::gov::cca::AbstractFramework > cxx_array_t;
    typedef ::gov::cca::AbstractFramework cxx_item_t;
    typedef struct gov_cca_AbstractFramework__array ior_array_t;
    typedef sidl_interface__array ior_array_internal_t;
    typedef struct gov_cca_AbstractFramework__object ior_item_t;
    typedef cxx_item_t value_type;
    typedef value_type reference;
    typedef value_type* pointer;
    typedef const value_type const_reference;
    typedef const value_type* const_pointer;
    typedef array_iter< array_traits< ::gov::cca::AbstractFramework > > 
      iterator;
    typedef const_array_iter< array_traits< ::gov::cca::AbstractFramework > > 
      const_iterator;
  };

  // array specialization
  template<>
  class array< ::gov::cca::AbstractFramework >: public interface_array< 
    array_traits< ::gov::cca::AbstractFramework > > {
  public:
    typedef interface_array< array_traits< ::gov::cca::AbstractFramework > > 
      Base;
    typedef array_traits< ::gov::cca::AbstractFramework >::cxx_array_t          
      cxx_array_t;
    typedef array_traits< ::gov::cca::AbstractFramework >::cxx_item_t           
      cxx_item_t;
    typedef array_traits< ::gov::cca::AbstractFramework >::ior_array_t          
      ior_array_t;
    typedef array_traits< ::gov::cca::AbstractFramework >::ior_array_internal_t 
      ior_array_internal_t;
    typedef array_traits< ::gov::cca::AbstractFramework >::ior_item_t           
      ior_item_t;

    /**
     * conversion from ior to C++ class
     * (constructor/casting operator)
     */
    array( struct gov_cca_AbstractFramework__array* src = 0) : Base(src) {}

    /**
     * copy constructor
     */
    array( const array< ::gov::cca::AbstractFramework >&src) : Base(src) {}

    /**
     * assignment
     */
    array< ::gov::cca::AbstractFramework >&
    operator =( const array< ::gov::cca::AbstractFramework >&rhs ) { 
      if (d_array != rhs._get_baseior()) {
        if (d_array) deleteRef();
        d_array = const_cast<sidl__array *>(rhs._get_baseior());
        if (d_array) addRef();
      }
      return *this;
    }

  };
}

#ifndef included_gov_cca_Services_hxx
#include "gov_cca_Services.hxx"
#endif
#ifndef included_gov_cca_TypeMap_hxx
#include "gov_cca_TypeMap.hxx"
#endif
#endif
